///////////////////////////////////////////////////////////////////////////
// Copyright 2000 Lucky Technolog Co., All rights reserved.              //
// Lucky Technology assumes no liability for errors in any of the        //
// software it provides.                                                 //
///////////////////////////////////////////////////////////////////////////
//
// This source code is only intended as a supplement to the
// iGuard API Software Development Kit and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// iGuard API Library product.
//
// LUCKY TECHNOLOGY CO., AND ITS LICENSORS MAKE NO WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, WITH REGARD TO THIS MATERIAL, INCLUDING, BUT
// NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//
///////////////////////////////////////////////////////////////////////////
// Source Code	Description
///////////////////////////////////////////////////////////////////////////
// exRspSvr.c:
//				1. Create a Window, initializes the iGuard API Library,
//				2. Grabs some information from an iGuard at IP Address:
//				   192.168.0.100 (firmware version and serial no.),
//				3. Start the response server and wait for the access log
//				   send from registered iGuard at IP Address 192.168.0.102
//				4. Show all the access log received from iGuard on screen
//				5. Shut down API Library when exits the program.
//
// Remarks:		To change the IP Addres of target iGuard, change the
//				definition of szURLAddress.
//
///////////////////////////////////////////////////////////////////////////
#include <windows.h>
#include "resource.h"
#include "..\Include\iGuard.h"			//iGuard API Library Hearder

#define MAX_LOADSTRING 4096				// Size of output buffer

// Global Variables:
HINSTANCE hInst;						// current instance
TCHAR szTitle[MAX_LOADSTRING];			// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];	// The title bar text
TCHAR szScreenBuffer[MAX_LOADSTRING];	// Message for Display
TCHAR szURLAddress[] = "192.168.0.102";	// Default IP Address of the iGuard Termianl
HWND hMainWnd;							// Windows handle

/////////////////////// DYNAMIC LOAD DLL /////////////////////////
//
// The following shows how to use LoadLibrary Function instead of
// using the "iguard.lib" file to link the iGuard.dll with 
// applications.
//////////////////////////////////////////////////////////////////

//#define _DYNAMIC_LOAD_	//Comment this line to use lib file.

typedef BOOL (*FUNC_EXIT) (void);
typedef BOOL (*FUNC_STOPSERVER) (void);
typedef BOOL (*FUNC_REGISTERSERVER) (LPCTSTR, BOOL);
typedef BOOL (*FUNC_LOGIN) (LPCTSTR, LPCTSTR, LPCTSTR);
typedef BOOL (*FUNC_STARTSERVER) (RecvAccessLogFunc *);
typedef BOOL (*FUNC_GETSERIALNO) (LPCTSTR, LPTSTR, int);
typedef BOOL (*FUNC_SHOWSERVERICON) (BOOL);
typedef BOOL (*FUNC_INITIALIZE) (HWND, BOOL, BOOL, DWORD);
typedef BOOL (*FUNC_GETVERSION) (LPCTSTR, LPTSTR, int);

#ifdef _DYNAMIC_LOAD_
		
HANDLE hiGuardDLL = NULL;				// iGuard.DLL

FUNC_EXIT iGuard_ExitA = NULL;
FUNC_STOPSERVER iGuard_StopServerA = NULL;
FUNC_REGISTERSERVER iGuard_RegisterServerA = NULL;
FUNC_LOGIN iGuard_LoginA = NULL;
FUNC_STARTSERVER iGuard_StartServerA = NULL;
FUNC_GETSERIALNO iGuard_GetSerialNoA = NULL;
FUNC_SHOWSERVERICON iGuard_ShowServerIconA = NULL;
FUNC_INITIALIZE iGuard_InitializeA = NULL;
FUNC_GETVERSION iGuard_GetVersionA = NULL;

#else //_DYNAMIC_LOAD_

FUNC_EXIT iGuard_ExitA = iGuard_Exit;
FUNC_STOPSERVER iGuard_StopServerA = iGuard_StopServer;
FUNC_REGISTERSERVER iGuard_RegisterServerA = iGuard_RegisterServer;
FUNC_LOGIN iGuard_LoginA = iGuard_Login;
FUNC_STARTSERVER iGuard_StartServerA = iGuard_StartServer;
FUNC_GETSERIALNO iGuard_GetSerialNoA = iGuard_GetSerialNo;
FUNC_SHOWSERVERICON iGuard_ShowServerIconA = iGuard_ShowServerIcon;
FUNC_INITIALIZE iGuard_InitializeA = iGuard_Initialize;
FUNC_GETVERSION iGuard_GetVersionA = iGuard_GetVersion;

#endif //_DYNAMIC_LOAD_
/////////////////////// DYNAMIC LOAD DLL /////////////////////////

// Foward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);
void				iGuardConnect(HWND hWnd);
void __stdcall		RecvAccessLog(LPDB_TIMELOG pAccessLog);

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_EXAMPLE, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	
	
	/////////////////////// DYNAMIC LOAD DLL /////////////////////////
#ifdef _DYNAMIC_LOAD_
	hiGuardDLL = LoadLibrary("iGuard.dll");

	if(hiGuardDLL == NULL) {
		MessageBox(NULL, "Error loading iGuard API", "iGuard Security System", MB_OK |  MB_ICONEXCLAMATION);
		return FALSE;
	} else {
		iGuard_ExitA = (FUNC_EXIT) GetProcAddress(hiGuardDLL, "iGuard_Exit");
		iGuard_StopServerA = (FUNC_STOPSERVER) GetProcAddress(hiGuardDLL, "iGuard_StopServer");
		iGuard_RegisterServerA = (FUNC_REGISTERSERVER) GetProcAddress(hiGuardDLL, "iGuard_RegisterServer");;
		iGuard_LoginA = (FUNC_LOGIN) GetProcAddress(hiGuardDLL, "iGuard_Login");
		iGuard_StartServerA = (FUNC_STARTSERVER) GetProcAddress(hiGuardDLL, "iGuard_StartServer");
		iGuard_GetSerialNoA = (FUNC_GETSERIALNO) GetProcAddress(hiGuardDLL, "iGuard_GetSerialNo");
		iGuard_ShowServerIconA = (FUNC_SHOWSERVERICON) GetProcAddress(hiGuardDLL, "iGuard_ShowServerIcon");
		iGuard_InitializeA = (FUNC_INITIALIZE) GetProcAddress(hiGuardDLL, "iGuard_Initialize");
		iGuard_GetVersionA = (FUNC_GETVERSION) GetProcAddress(hiGuardDLL, "iGuard_GetVersion");

		if(!(iGuard_ExitA && iGuard_StopServerA && iGuard_RegisterServerA && 
			iGuard_LoginA && iGuard_StartServerA && iGuard_GetSerialNoA && 
			iGuard_ShowServerIconA && iGuard_InitializeA && iGuard_GetVersionA)) {
			MessageBox(NULL, "Error loading iGuard API", "iGuard Security System", MB_OK |  MB_ICONEXCLAMATION);
			return FALSE;
		}
	}
#endif //_DYNAMIC_LOAD_
	/////////////////////// DYNAMIC LOAD DLL /////////////////////////

	// Perform application initialization:
	if(!InitInstance (hInstance, nCmdShow)) {
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_EXAMPLE);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	/////////////////////// DYNAMIC LOAD DLL /////////////////////////
#ifdef _DYNAMIC_LOAD_
	FreeLibrary((HMODULE) hiGuardDLL);
#endif //_DYNAMIC_LOAD_
	/////////////////////// DYNAMIC LOAD DLL /////////////////////////

	return msg.wParam;
}

ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_EXAMPLE);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCSTR) IDC_EXAMPLE;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
   HWND hWnd;

   hInst = hInstance; // Store instance handle in our global variable

   hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!hWnd) {
      return FALSE;
   }

   ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);

   return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	RECT rt;
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;
	HFONT hFont;

	switch (message) {
		case WM_CREATE:
			hMainWnd = hWnd;
			LoadString(hInst, IDS_HELLO, szScreenBuffer, MAX_LOADSTRING);
			break;

		case WM_USER + 1:
			switch(lParam) {
				case IGM_STARTSERVER:
					//
					// Handling for response server start
					//////////////////////////////////////
					break;

				case IGM_STOPSERVER:
					//
					// Handling for response server stop
					//////////////////////////////////////
					break;

				default:
					break;
			}
			break;

		case WM_COMMAND:
			wmId    = LOWORD(wParam);
			wmEvent = HIWORD(wParam);
			// Parse the menu selections:
			switch (wmId) {
				case IDM_START:
					iGuardConnect(hWnd);	//iGuard API Example
					break;

				case IDM_ABOUT:
				   	DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
				   	break;

				case IDM_EXIT:
				   	DestroyWindow(hWnd);
				   	break;

				default:
				   	return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;

		case WM_PAINT:
			//
			// Write screen buffer to screen
			//////////////////////////////////////
			hdc = BeginPaint(hWnd, &ps);
			hFont = CreateFont(-12, 0, 0, 0, FW_LIGHT, FALSE, FALSE, FALSE,
				ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
					DEFAULT_QUALITY, DEFAULT_PITCH | FF_SWISS, "Verdana");
			SelectObject(hdc, hFont);
			GetClientRect(hWnd, &rt);
			rt.left += 5;
			DrawText(hdc, szScreenBuffer, strlen(szScreenBuffer), &rt, DT_LEFT);
			DeleteObject(hFont);
			EndPaint(hWnd, &ps);
			break;

		case WM_DESTROY:
			//
			// iGuard API Shutdown Procedure
			//////////////////////////////////////
			iGuard_StopServerA();	//Stop Response Server
			iGuard_ExitA();			//Shutdown and Clean up API library
			PostQuitMessage(0);
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

// Mesage handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		case WM_INITDIALOG:
				return TRUE;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}

//
// Subroutine for demostrate how to initialize and
// use the iGuard API Software Development Kit
////////////////////////////////////////////////////////////
void iGuardConnect(HWND hWnd)
{
	TCHAR szBuffer[100];
	TCHAR szOutput[100];

	//
	// Initial the API Library
	//////////////////////////////////////
	if(iGuard_InitializeA(hWnd, TRUE, TRUE, WM_USER + 1)) {
		
		strcat(szScreenBuffer, "\niGuard API Started.");

		//
		// Show API's Tray icon
		//////////////////////////////////////
		iGuard_ShowServerIconA(TRUE);

		//
		// Get version from an iGuard
		//////////////////////////////////////
		wsprintf(szOutput, "\nConnecting to iGuard %s...", szURLAddress);
		strcat(szScreenBuffer, szOutput);

		if(iGuard_GetVersionA(szURLAddress, szBuffer, 100)) {

			wsprintf(szOutput, "\niGuard %s Connected...\nFirmware Version : %s",
				szURLAddress, szBuffer);
			strcat(szScreenBuffer, szOutput);

			//
			// Get Serial No. from an iGuard
			//////////////////////////////////////
			if(iGuard_GetSerialNoA(szURLAddress, szBuffer, 100)) {
				
				wsprintf(szOutput, "\nSerial No. : %s",
					szBuffer);
				strcat(szScreenBuffer, szOutput);

				//
				// Start iGuard Response Server
				//////////////////////////////////////
				if(iGuard_StartServerA((RecvAccessLogFunc *)RecvAccessLog)) {
					strcat(szScreenBuffer, "\nResponse Server Started.");

					//
					// Setup Login Information of iGuard Terminal
					// *** must login for security reason ***
					///////////////////////////////////////////////////////
					iGuard_LoginA(szURLAddress, "Admin", "123");

					//
					// Register the Response Server to an iGuard Terminal
					///////////////////////////////////////////////////////
					if(iGuard_RegisterServerA(szURLAddress, TRUE)) {
						strcat(szScreenBuffer, "\nRegistered Response Server to iGuard.\n\nWaiting for Access Log...\n(Format: \"ID MM/DD/YYYY HH:MM:SS In/Out Terminal\")\n-------------------------------------------");

					} else {
						strcat(szScreenBuffer, "\nResponse Server Register Failed.");

					}

				} else {
					strcat(szScreenBuffer, "\nResponse Server Start Failed.");

				}

			} else {
				strcat(szScreenBuffer, "\nGet Serial No. Failed.");

			}

		} else {
			strcat(szScreenBuffer, "\nGet Firmware Version Failed.");

		}

	} else {
		strcat(szScreenBuffer, "\niGuard API Start Failed.");

	}

	InvalidateRect(hWnd, NULL, FALSE);	//Tell window to redraw the screen
}

//
// CallBack function for receiving log from iGuard
////////////////////////////////////////////////////////
void __stdcall RecvAccessLog(LPDB_TIMELOG pAccessLog)
{
	TCHAR szBuffer[100];
	SYSTEMTIME sysTime;

	//
	// Format and display received access log on screen
	//////////////////////////////////////////////////////
	FileTimeToSystemTime(&pAccessLog->logTime, &sysTime);

	wsprintf(szBuffer, "\n%s %d/%d/%d %02d:%02d:%02d %s %s", pAccessLog->id,
		sysTime.wMonth, sysTime.wDay, sysTime.wYear,
			sysTime.wHour, sysTime.wMinute, sysTime.wSecond,
				(pAccessLog->status & 1) == 0 ? "IN" : "OUT",
					pAccessLog->terminalID);

	strcat(szScreenBuffer, szBuffer);
	InvalidateRect(hMainWnd, NULL, FALSE);

	if(strlen(szScreenBuffer) - MAX_LOADSTRING < 50) {
		memset(szScreenBuffer, '\0', MAX_LOADSTRING);
	}
}
